
	.showlog { font-family: arial,serif; background-color: white; color: black; }

	.errorMessage { font-family: arial,serif; text-align: center; color: red; font-weight: bold; font-size: 12pt; }
	.errorDescription { font-family: arial,serif; text-align: center; font-weight: bold; font-size: 12pt; }
	.warningMessage { font-family: arial,serif; text-align: center; color: red; font-weight: bold; font-size: 10pt; }

	.infoBox { font-family: arial,serif; font-size: 8pt; background-color: #C4C2C2; padding: 2; }
	.infoBoxTitle { font-family: arial,serif; font-size: 10pt; font-weight: bold; }
	.infoBoxBadProcStatus { color: red; }
	A.homepageURL:Hover { font-family: arial,serif; color: red; }
	
	.navBoxTitle { font-family: arial narrow,serif; font-size: 10pt; font-weight: bold; }
	.navBoxItem { font-family: arial,serif; font-size: 8pt; font-weight: bold; color: red; }
	.navBoxDate { font-family: arial,serif; font-size: 8pt; font-weight: bold; }
	.navBoxFile { font-family: arial,serif; font-size: 8pt; font-weight: bold; text-align: center; }
	
	.optBox { font-family: arial narrow,serif; font-size: 10pt; font-weight: bold; }
	.optBoxItem { font-family: arial,serif; font-size: 8pt; font-weight: bold; color: red; }

	.logEntries { font-family: arial,serif; font-size: 8pt; }
	
	.dateTimeBreak { font-family: arial,serif; font-size: 10pt; font-weight: bold; color: red; }
	
